import { r as registerInstance, c as createEvent, h, g as getElement } from './p-597600b0.js';
import { GcUtils } from '../../components/@ti/gc-core-assets/lib/GcUtils';
import { DataConverter, valueChangedEventType, bindingRegistry } from '../../components/@ti/gc-core-databind/lib/CoreDatabind';
import { codecRegistry } from '../../components/@ti/gc-target-configuration/lib/TargetConfiguration';

const gcCustomRegisterBitfieldViewerCss = ".container,.info-container:not([hidden]){display:flex;flex-direction:column;width:100%}.container{height:100%;box-sizing:border-box}.heading{font-size:21px;font-weight:500;position:relative}.heading:before{content:\"\";position:absolute;width:calc(100% - 10px);height:1px;bottom:2px;border-bottom:2px solid #cc0000}.sub-heading{font-size:14px}.info-title,.sub-heading{color:#118899;font-weight:bold}.info-container>div.info-title{margin-bottom:4px;margin-top:10px}.info-container,.info-container gc-widget-label{user-select:text}.heading,.sub-heading,.scroll-container{padding:5px}.sub-heading-default{color:#555555;font-weight:500;padding-top:10px}.scroll-container{overflow:auto}.field-container *,.info-container *{margin:0px}.field-container{align-items:center;padding-top:5px}.field-container:not(:nth-last-child(2)){border-bottom:1px solid #e5e5e5;padding-bottom:5px}.header-container,.widget-container{display:flex;flex-direction:row;align-items:center}.header-container>*:not(:last-child),.widget-container>*:not(:last-child){margin-right:5px}.header-container{margin-bottom:4px}.header-container gc-widget-icon{--gc-color:#98999b;cursor:pointer}.header-container div.caption{color:#999999;font-size:14px;font-weight:400}.widget-container{min-height:34px;}.widget-container gc-widget-led:focus{outline:1px dotted #555555;outline-offset:-1px}.flex{flex-grow:1;flex-shrink:1}.right-align{--gc-text-align:right}:host{user-select:none;font-family:\"Roboto\", \"Open Sans\", \"Segoe UI\", Tahoma, sans-serif;font-size:14px;font-weight:400;color:var(--gc-font-color, var(--theme-font-color, #555555));margin:5px;display:inline-block;-webkit-box-sizing:border-box;box-sizing:border-box}.registerDetails-view{justify-content:center;align-items:center;display:flex;color:grey}";

const WidgetRegisterBitfieldViewer = class {
  constructor(hostRef) {
    registerInstance(this, hostRef);
    this.valueChanged = createEvent(this, "value-changed", 7);
    this.selectedFieldIndexChanged = createEvent(this, "selected-field-index-changed", 7);
    this.cssPropertyChanged = createEvent(this, "css-property-changed", 7);
    this.keyCounters = new Map();
    this.isRendering = false;
    this.onRegisterValueChangedListener = () => {
      var _a, _b;
      this.value =
        (_b = (_a = this.registerValueBind) === null || _a === void 0 ? void 0 : _a.getValue()) !== null && _b !== void 0 ? _b : this.registerInfo.default;
    };
    this.onRegisterSymbolsChangedListener = () => {
      var _a;
      this.registerInfo = (_a = this.registerModel) === null || _a === void 0 ? void 0 : _a.getRegisterInfo(this.registerName);
      this.selectedFieldIndex = -1;
    };
    this.onIndicatorToggle = (e) => {
      var _a, _b;
      const bitFieldIndex = +e.currentTarget.id;
      const bitField = (_b = (_a = this.registerInfo) === null || _a === void 0 ? void 0 : _a.fields) === null || _b === void 0 ? void 0 : _b[bitFieldIndex];
      if (bitField && !this.isReadOnly(bitField)) {
        this.setBitFieldValue(bitField, this.getBitFieldValue(bitField) ? 0 : 1);
      }
    };
    this.onIndicatorKeyHandler = (e) => {
      if (e.key === " " && !e.shiftKey && !e.ctrlKey) {
        this.onIndicatorToggle(e);
        e.preventDefault();
        e.stopPropagation();
      }
    };
    this.toggleHelp = (e) => {
      if (this.selectedFieldIndex < 0) {
        this.selectedFieldIndex = +e.currentTarget.id;
      }
      else {
        this.selectedFieldIndex = -1;
      }
    };
    this.onWidgetValueChanged = (e) => {
      var _a, _b;
      const bitFieldIndex = +e.currentTarget.id;
      const bitField = (_b = (_a = this.registerInfo) === null || _a === void 0 ? void 0 : _a.fields) === null || _b === void 0 ? void 0 : _b[bitFieldIndex];
      if (bitField) {
        this.setBitFieldValue(bitField, +e.detail.value);
      }
    };
    this.registerModel = undefined;
    this.registerInfo = undefined;
    this.heading = undefined;
    this.registerName = undefined;
    this.registerModelId = undefined;
    this.selectedFieldIndex = -1;
    this.value = 0;
    this._refresh = 0;
    this.hidden = false;
  }
  getBitFieldType(bitField) {
    var _a, _b, _c, _d, _e;
    if (((_a = bitField.attrs) === null || _a === void 0 ? void 0 : _a.isLocked) || ((_c = (_b = this.registerInfo) === null || _b === void 0 ? void 0 : _b.attrs) === null || _c === void 0 ? void 0 : _c.readonly)) {
      return "R";
    }
    if ((_e = (_d = this.registerInfo) === null || _d === void 0 ? void 0 : _d.attrs) === null || _e === void 0 ? void 0 : _e.writeonly) {
      return "W";
    }
    return "RW";
  }
  isReadOnly(bitField) {
    return this.getBitFieldType(bitField) === "R";
  }
  getBitFieldRange(bitField, separator = ":") {
    if (bitField === undefined) {
      return "";
    }
    if (bitField.stop > bitField.start) {
      return `${bitField.stop}${separator}${bitField.start}`;
    }
    if (bitField.stop === bitField.start) {
      return `${bitField.stop}`;
    }
    return "";
  }
  getSelectValues(bitField) {
    if (bitField.options) {
      return `${bitField.options.map(option => option.value).join("|")}|`;
    }
    const width = bitField.stop - bitField.start + 1;
    const numOptions = 1 << width;
    return `${new Array(numOptions)
      .fill("")
      .map((_, i) => `${i}`)
      .join("|")}|`;
  }
  getSelectLabels(bitField) {
    if (bitField.options) {
      return `${bitField.options.map(option => option.display).join("|")}|`;
    }
    const width = bitField.stop - bitField.start + 1;
    const numOptions = 1 << width;
    return `${new Array(numOptions)
      .fill("")
      .map((_, i) => `b${DataConverter.convert(i, "number", "binary", width)}`)
      .join("|")}|`;
  }
  isDecimalFormat(bitField) {
    return (bitField.widget &&
      (bitField.widget.format === "dec" ||
        bitField.widget.format === "exp" ||
        (!bitField.widget.format && bitField.widget.type === "spinner")));
  }
  getBitFieldValue(bitField) {
    if (this.isDecimalFormat(bitField)) {
      // decimal values could be signed, fixed point, or have getters and setters, so use register model to get the value.
      return this.registerModel
        .getBinding(`${this.registerName}.${this.getSymbolName(bitField)}`)
        .getValue();
    }
    // simple binary and hexadecimal values don't care about signed, decimal point or getters and setters, and we just need raw value.
    const mask = GcUtils.bitField.getMask(bitField.start, bitField.stop);
    return GcUtils.bitField.readField(this.value, mask, bitField.start);
  }
  setBitFieldValue(bitField, value) {
    if (this.isRendering) {
      // during rendering we are reusing widgets, so we expect to have widget change listeners firing, and they need to be ignored.
      return;
    }
    if (this.isDecimalFormat(bitField)) {
      // decimal values could be signed, fixed point, or have getters and setters, so use register model to set the value.
      this.registerModel
        .getBinding(`${this.registerName}.${this.getSymbolName(bitField)}`)
        .setValue(value);
    }
    else {
      // simple binary and hexadecimal values don't care about signed, decimal point or getters and setters, and we just need to set the raw value.
      const mask = GcUtils.bitField.getMask(bitField.start, bitField.stop);
      // @Watch on value will commit the new value to the register binding.
      this.value = GcUtils.bitField.writeField(this.value, mask, bitField.start, value);
    }
  }
  getFormat(bitField) {
    var _a, _b, _c;
    return ((_b = (_a = bitField.widget) === null || _a === void 0 ? void 0 : _a.format) !== null && _b !== void 0 ? _b : (((_c = bitField.widget) === null || _c === void 0 ? void 0 : _c.type) === "spinner" ? "dec" : "hex"));
  }
  getPrecision(bitField) {
    var _a, _b, _c, _d;
    if (((_a = bitField.widget) === null || _a === void 0 ? void 0 : _a.type) === "spinner" && !bitField.widget.format) {
      let decimalPlaces = 0;
      const incrementString = (_c = (_b = bitField.widget.step) === null || _b === void 0 ? void 0 : _b.toString()) !== null && _c !== void 0 ? _c : "";
      if (incrementString.includes(".")) {
        decimalPlaces =
          incrementString.length - incrementString.indexOf(".") - 1;
      }
      return decimalPlaces;
    }
    return ((_d = bitField.widget) === null || _d === void 0 ? void 0 : _d.format)
      ? bitField.widget.precision
      : Math.ceil((bitField.stop - bitField.start + 1) / 4);
  }
  getReadOnlyDisplayValue(bitField, bitFieldValue) {
    var _a;
    if (bitField.options && bitField.options.length > bitFieldValue) {
      return bitField.options[bitFieldValue].display;
    }
    const width = bitField.stop - bitField.start + 1;
    let format;
    let precision;
    if ((_a = bitField.widget) === null || _a === void 0 ? void 0 : _a.format) {
      // user provided format and precision (assumed, because undefined precision is allowed)
      format = bitField.widget.format;
      precision = bitField.widget.precision;
    }
    else {
      // default format and precision
      format = width < 8 ? "binary" : "hex"; // This is from V2, where read only values, less that 8 bits, were displayed in boolean.
      precision = width < 8 ? width : Math.ceil(width / 4);
    }
    const displayValue = DataConverter.convert(bitFieldValue, "number", format, precision);
    return format === "binary" ? "b" + displayValue : displayValue;
  }
  getSymbolName(info) {
    return info.id || info.name;
  }
  getRegisterDisplayName() {
    return this.registerInfo
      ? `${this.registerInfo.groupName ? `${this.registerInfo.groupName} / ` : ""}${this.registerInfo.name}`
      : "";
  }
  getDefaultRegisterValue() {
    let defaultVal = this.registerInfo
      ? this.registerInfo.groupName
        ? parseInt(this.registerInfo.default.toString(16), 16)
          .toString(16)
          .toUpperCase()
          .padStart(2, "0")
        : 0
      : 0;
    return this.registerInfo
      ? this.registerInfo.groupName
        ? `Default Value: 0x${defaultVal}`
        : ""
      : "";
  }
  getBifFieldWidgetType(bitField) {
    var _a, _b;
    const readonly = this.isReadOnly(bitField);
    const isWidget = ((_a = bitField.widget) === null || _a === void 0 ? void 0 : _a.type) !== undefined && bitField.widget.type !== "value";
    if (readonly && !isWidget) {
      return "label";
    }
    if ((_b = bitField.widget) === null || _b === void 0 ? void 0 : _b.type) {
      return bitField.widget.type;
    }
    if (bitField.options) {
      return "select";
    }
    if (bitField.stop === bitField.start) {
      // 1 bit
      return "checkbox";
    }
    if (bitField.stop <= bitField.start + 3) {
      // 2-4 bits
      return "select";
    }
    return "value"; // more than 4 bits
  }
  generateKey(type) {
    let counter = this.keyCounters.get(type);
    if (counter === undefined) {
      counter = { count: 0, total: 0 };
      this.keyCounters.set(type, counter);
    }
    counter.count++;
    if (counter.count > counter.total) {
      counter.total = counter.count;
    }
    return `${type}${counter.count}`;
  }
  renderBitFieldWidget(type, readonly, id = undefined, bitField) {
    var _a, _b, _c, _d, _e, _f;
    const value = bitField ? this.getBitFieldValue(bitField) : 0;
    const align = ((_a = bitField === null || bitField === void 0 ? void 0 : bitField.widget) === null || _a === void 0 ? void 0 : _a.units) ? "flex right-align" : "flex";
    switch (type) {
      case "label":
        return (h("gc-widget-label", { id: id, label: bitField
            ? this.getReadOnlyDisplayValue(bitField, value)
            : undefined }));
      case "checkbox":
        return (h("gc-widget-checkbox", { id: id, readonly: readonly, checked: value !== 0, "onChecked-changed": this.onWidgetValueChanged }));
      case "select":
        return (h("gc-widget-select", { id: id, class: align, readonly: readonly, "selected-value": value, values: bitField ? this.getSelectValues(bitField) : "0", labels: bitField ? this.getSelectLabels(bitField) : "_", "onSelected-value-changed": this.onWidgetValueChanged }));
      case "spinner":
        return (h("gc-widget-spinner", { id: id, class: "flex right-align", readonly: readonly, value: value, "min-value": (_b = bitField === null || bitField === void 0 ? void 0 : bitField.widget) === null || _b === void 0 ? void 0 : _b.min, "max-value": (_c = bitField === null || bitField === void 0 ? void 0 : bitField.widget) === null || _c === void 0 ? void 0 : _c.max, increment: (_d = bitField === null || bitField === void 0 ? void 0 : bitField.widget) === null || _d === void 0 ? void 0 : _d.step, format: bitField ? this.getFormat(bitField) : "dec", precision: bitField ? this.getPrecision(bitField) : undefined, "onValue-changed": this.onWidgetValueChanged }));
      case "indicator":
        return (h("gc-widget-led", { id: id, on: value !== 0, onClick: this.onIndicatorToggle, tabIndex: 0, style: {
            "--gc-on-color": (_e = bitField === null || bitField === void 0 ? void 0 : bitField.widget) === null || _e === void 0 ? void 0 : _e.onColor,
            "--gc-off-color": (_f = bitField === null || bitField === void 0 ? void 0 : bitField.widget) === null || _f === void 0 ? void 0 : _f.offColor,
          }, onKeyDown: this.onIndicatorKeyHandler }));
      default:
        return (h("gc-widget-input", { id: id, class: align, readonly: readonly, value: value, format: bitField ? this.getFormat(bitField) : "hex", precision: bitField ? this.getPrecision(bitField) : 4, "onValue-changed": this.onWidgetValueChanged }));
    }
  }
  renderBitField(type, bitFieldIndex = -1, bitField) {
    var _a, _b;
    const readonly = bitField ? this.isReadOnly(bitField) : false;
    const id = bitFieldIndex < 0 ? undefined : `${bitFieldIndex}`;
    const isSelected = bitFieldIndex < 0 ? false : bitFieldIndex === this.selectedFieldIndex;
    const hidden = bitFieldIndex < 0 || (!isSelected && this.selectedFieldIndex >= 0);
    const key = this.generateKey(type);
    return (h("div", { class: "field-container", key: key, id: key, hidden: hidden }, h("div", { class: "header-container" }, h("div", { class: "caption" }, `${(_a = bitField === null || bitField === void 0 ? void 0 : bitField.name) !== null && _a !== void 0 ? _a : ""}[${this.getBitFieldRange(bitField)}]`), h("gc-widget-icon", { id: id, appearance: isSelected ? "primary" : "custom", icon: isSelected ? "navigation:cancel" : "action:help_outline", size: isSelected ? "s" : "xs", onClick: this.toggleHelp })), h("div", { class: "widget-container" }, h("gc-widget-icon", { hidden: !readonly, size: "xs", appearance: "tertiary", icon: "action:lock" }), this.renderBitFieldWidget(type, readonly, id, bitField), ((_b = bitField === null || bitField === void 0 ? void 0 : bitField.widget) === null || _b === void 0 ? void 0 : _b.units) ? (h("gc-widget-label", { label: bitField.widget.units })) : null)));
  }
  render() {
    var _a, _b, _c, _d, _e, _f;
    const bitField = (_b = (_a = this.registerInfo) === null || _a === void 0 ? void 0 : _a.fields[this.selectedFieldIndex]) !== null && _b !== void 0 ? _b : { name: "", start: 0, stop: 0 };
    const hideInfo = this.selectedFieldIndex < 0;
    // reset key counters per widget type
    this.keyCounters.forEach(counter => (counter.count = 0));
    // create bit field containers, and increment key counters in the process
    const bitFieldContainers = (_e = (_d = (_c = this.registerInfo) === null || _c === void 0 ? void 0 : _c.fields) === null || _d === void 0 ? void 0 : _d.map((bitField, i) => {
      var _a;
      return ((_a = bitField.attrs) === null || _a === void 0 ? void 0 : _a.isHidden)
        ? null
        : this.renderBitField(this.getBifFieldWidgetType(bitField), i, bitField);
    })) !== null && _e !== void 0 ? _e : [];
    // create extra hidden field containers to prevent destruction of field widgets, to improve performance.
    const extraHiddenFieldContainers = [];
    this.keyCounters.forEach((counter, type) => {
      while (counter.count < counter.total) {
        extraHiddenFieldContainers.push(this.renderBitField(type));
      }
    });
    return (h("div", { class: "container" }, this.heading ? (h("div", { key: "heading", class: "heading" }, this.heading)) : null, h("div", { key: "sub-heading", class: "sub-heading" }, h("div", { class: "registerDetails-view" }, this.registerInfo
      ? ""
      : `Note: Select a register to view it's register details`), this.getRegisterDisplayName(), h("div", { class: "sub-heading-default" }, this.getDefaultRegisterValue())), h("div", { key: "scroll-container", class: "scroll-container" }, extraHiddenFieldContainers, bitFieldContainers, hideInfo ? null : h("div", { key: "forceDividerLine" }), h("div", { class: "info-container", key: "information", hidden: hideInfo }, h("div", { class: "info-title" }, "Field"), h("gc-widget-label", { label: bitField.title || bitField.name }), h("div", { class: "info-title" }, "Bit"), h("gc-widget-label", { label: this.getBitFieldRange(bitField, " - ") }), h("div", { class: "info-title" }, "Type"), h("gc-widget-label", { label: this.getBitFieldType(bitField) }), h("div", { class: "info-title" }, "Description"), h("div", { innerHTML: (_f = bitField.desc) === null || _f === void 0 ? void 0 : _f.split("\n").join("<br>") })))));
  }
  addRegisterBindListener() {
    if (this.registerModel) {
      this.registerModel
        .getBinding("$registerDefinitions")
        .addEventListener(valueChangedEventType, this.onRegisterSymbolsChangedListener);
      if (this.registerInfo) {
        this.registerValueBind = this.registerModel.getBinding(this.getSymbolName(this.registerInfo));
        this.registerValueBind.addEventListener(valueChangedEventType, this.onRegisterValueChangedListener);
      }
    }
  }
  removeRegisterBindListener() {
    if (this.registerValueBind) {
      this.registerValueBind.removeEventListener(valueChangedEventType, this.onRegisterValueChangedListener);
      this.registerValueBind = undefined;
    }
    if (this.registerModel) {
      this.registerModel
        .getBinding("$registerDefinitions")
        .removeEventListener(valueChangedEventType, this.onRegisterSymbolsChangedListener);
    }
  }
  connectedCallback() {
    this.onRegisterInfoChanged();
  }
  disconnectedCallBack() {
    this.removeRegisterBindListener();
  }
  componentWillRender() {
    this.isRendering = true;
  }
  componentDidRender() {
    this.isRendering = false;
  }
  onValueChanged() {
    var _a;
    (_a = this.registerValueBind) === null || _a === void 0 ? void 0 : _a.setValue(this.value);
    this.valueChanged.emit({ value: this.value });
  }
  onRegisterInfoChanged() {
    this.removeRegisterBindListener();
    let model;
    if (this.registerModelId) {
      model = bindingRegistry.getModel(this.registerModelId);
      if (!model && this.registerModelId) {
        codecRegistry
          .whenConfigurationReady(this.registerModelId)
          .then(() => this.onRegisterInfoChanged());
      }
    }
    this.registerModel = (model === null || model === void 0 ? void 0 : model.getConvertedValue) ? model : undefined;
    this.onRegisterSymbolsChangedListener();
    this.addRegisterBindListener();
    if (this.registerInfo) {
      this.onRegisterValueChangedListener();
    }
  }
  onSelectedFieldChanged() {
    this.selectedFieldIndexChanged.emit({ value: this.selectedFieldIndex });
  }
  /**
   * Sets the CSS property.
   *
   * @param {string} name the element style name
   * @param {string} value the new CSS property to be set
   */
  async setCSSProperty(name, value) {
    value = value.trim();
    if ((await this.getCSSProperty(name)) !== value) {
      this.el.style.setProperty(name, value);
      this.cssPropertyChanged.emit({ name: name, value: value });
    }
  }
  /**
   * Returns the value of a CSS property.
   *
   * @param {string} name the element style property
   * @returns {string} the value of the property
   */
  async getCSSProperty(name) {
    return getComputedStyle(this.el).getPropertyValue(name).trim();
  }
  /**
   * Refresh the widget.
   */
  async refresh() {
    this._refresh++;
  }
  get el() { return getElement(this); }
  static get watchers() { return {
    "value": ["onValueChanged"],
    "registerModelId": ["onRegisterInfoChanged"],
    "registerName": ["onRegisterInfoChanged"],
    "selectedFieldIndex": ["onSelectedFieldChanged"]
  }; }
};
WidgetRegisterBitfieldViewer.style = gcCustomRegisterBitfieldViewerCss;

const gcCustomRegisterGridCss = "gc-widget-grid-tree-column{--gc-column-width:minmax(220px, 1fr)}gc-widget-grid-data-column{--gc-text-align:center}:host{border:2px solid #e5e5e5}#grid{width:100%;height:100%;margin:0px;border:none}#grid::part(open){font-style:italic}#grid::part(branch){font-weight:bold;font-size:93%}#grid::part(leaf){font-size:86%}#bitsHeader::part(bit-cell){border:1px solid #ffffff;border-bottom-width:0px;border-right-width:0px}#bitsHeader::part(bit-cell first-child){border-left-color:transparent}#bitsHeader::part(bit-cell),#grid::part(bit-cell){width:100%;display:inline-flex;align-items:center;justify-content:center}#grid::part(bit-cell readonly){background-color:#e5e5e5;color:#999999}#grid::part(bit-cell selected){background-color:#d9f1f4}#grid::part(deferred){text-decoration:underline;font-style:italic}#grid::part(bits-container),#bitsHeader::part(bits-container){grid-gap:0;grid-template-rows:1fr;justify-items:center;height:100%}#grid::part(bits-container){padding-right:1px;padding-left:2px}#bitsHeader::part(bits-header-padding){overflow:hidden;width:0px;display:none;}";

class RegisterData {
  constructor(regInfo, registerModel) {
    var _a, _b, _c;
    this.reservedMask = 0;
    this.isVisibleInGrid = false;
    this.name = regInfo.name || regInfo.id;
    this.address = regInfo.addr;
    this.binding = registerModel.getBinding(regInfo.id);
    this.readonly = (_b = (_a = regInfo.attrs) === null || _a === void 0 ? void 0 : _a.readonly) !== null && _b !== void 0 ? _b : false;
    this.path = `${regInfo.groupName}/${regInfo.id}`;
    this.visibleMask = GcUtils.bitField.getMask(0, (regInfo.size || regInfo.nBytes * 8 || 8) - 1);
    this.readonlyMask = this.readonly ? this.visibleMask : 0;
    (_c = regInfo.fields) === null || _c === void 0 ? void 0 : _c.forEach(field => {
      if (field.attrs) {
        const mask = GcUtils.bitField.getMask(field.start, field.stop);
        if (field.attrs.isHidden) {
          this.visibleMask = GcUtils.bitField.writeField(this.visibleMask, mask, field.start, 0);
        }
        if (field.attrs.isLocked) {
          this.readonlyMask = GcUtils.bitField.writeField(this.readonlyMask, mask, field.start, -1);
        }
        if (field.attrs.isReserved) {
          this.reservedMask = GcUtils.bitField.writeField(this.reservedMask, mask, field.start, -1);
        }
      }
    });
  }
  get value() {
    return this.binding.getValue();
  }
  set value(newValue) {
    if (!this.readonly) {
      this.binding.setValue(newValue);
    }
  }
  get status() {
    return this.binding.status;
  }
  get committedValue() {
    return this.binding.getValueCommitted();
  }
  get isDeferredWritePending() {
    return this.binding.isDeferredWritePending();
  }
}
class RegisterBlockDataProvider {
  constructor(group, registerModel) {
    this.name = group.name;
    this.registers = group.registers
      .filter(regInfo => { var _a; return !((_a = regInfo.attrs) === null || _a === void 0 ? void 0 : _a.isHidden); })
      .map(regInfo => new RegisterData(regInfo, registerModel));
  }
  addChangeListeners(onChangedCallback) {
    this.registers.forEach(register => register.binding.addEventListener(valueChangedEventType, onChangedCallback));
  }
  removeChangeListeners(onChangedCallback) {
    this.registers.forEach(register => register.binding.removeEventListener(valueChangedEventType, onChangedCallback));
  }
  get rowCount() {
    return this.registers.length;
  }
  getValue(row, column) {
    return this.registers[row][column];
  }
  setValue(row, column, value) {
    this.registers[row][column] = value;
  }
  isReadonly(row) {
    return this.registers[row].readonly;
  }
  lookupRowByName(name) {
    return this.registers.findIndex(register => register.name === name);
  }
  lookupRegisterByName(name) {
    return this.registers[this.lookupRowByName(name)];
  }
  getPartName(row, column) {
    return column === "value" && this.registers[row].isDeferredWritePending
      ? "deferred"
      : "";
  }
}
class RegisterBlockVirtualDataProvider extends RegisterBlockDataProvider {
  updateChangedEventListener(register) {
    if (this.onChangedEventListener) {
      if (register.isVisibleInGrid) {
        register.binding.addEventListener(valueChangedEventType, this.onChangedEventListener);
      }
      else {
        register.binding.removeEventListener(valueChangedEventType, this.onChangedEventListener);
      }
    }
  }
  setVisibility(row, visible) {
    const register = this.registers[row];
    if (!register) {
      return;
    }
    if (register.isVisibleInGrid === visible) {
      return;
    }
    register.isVisibleInGrid = visible;
    this.updateChangedEventListener(register);
  }
  addChangeListeners(onChangedCallback) {
    this.onChangedEventListener = onChangedCallback;
    this.registers.forEach(register => this.updateChangedEventListener(register));
  }
  removeChangeListeners(onChangedCallback) {
    this.onChangedEventListener = undefined;
    super.removeChangeListeners(onChangedCallback);
  }
}
class RegisterGridDataProvider {
  constructor(registerModel, virtual) {
    var _a, _b;
    const deviceData = (_a = registerModel === null || registerModel === void 0 ? void 0 : registerModel.getBinding("$registerDefinitions")) === null || _a === void 0 ? void 0 : _a.getValue();
    this.dataBits = (deviceData === null || deviceData === void 0 ? void 0 : deviceData.info.regsize) || 16;
    const biggestAddress = (deviceData === null || deviceData === void 0 ? void 0 : deviceData.regblocks.reduce((max, block) => block.registers.reduce((max, reg) => Math.max(max, reg.addr), max), 0)) || 255;
    this.addressSize = biggestAddress.toString(16).length;
    this.groups =
      (_b = deviceData === null || deviceData === void 0 ? void 0 : deviceData.regblocks.filter(group => { var _a; return !((_a = group.attrs) === null || _a === void 0 ? void 0 : _a.isHidden); }).map(group => {
        if (virtual) {
          return new RegisterBlockVirtualDataProvider(group, registerModel);
        }
        return new RegisterBlockDataProvider(group, registerModel);
      })) !== null && _b !== void 0 ? _b : [];
  }
  isExpandable(row) {
    return true;
  }
  getChildDataProvider(row) {
    return this.groups[row];
  }
  get rowCount() {
    return this.groups.length;
  }
  getValue(row, column) {
    return this.groups[row][column];
  }
  setValue() { }
  lookupRowByName(name) {
    return this.groups.findIndex(group => group.name === name);
  }
  addChangeListeners(onChangedCallback) {
    this.groups.forEach(group => group.addChangeListeners(onChangedCallback));
  }
  removeChangeListeners(onChangedCallback) {
    this.groups.forEach(group => group.removeChangeListeners(onChangedCallback));
  }
  lookupRegisterByName(registerName) {
    if (!registerName) {
      return;
    }
    return this.groups.reduce((result, group) => result !== null && result !== void 0 ? result : group.lookupRegisterByName(registerName), undefined);
  }
}
const WidgetRegisterGrid = class {
  constructor(hostRef) {
    registerInstance(this, hostRef);
    this.selectedRegisterChanged = createEvent(this, "selected-register-changed", 7);
    this.cssPropertyChanged = createEvent(this, "css-property-changed", 7);
    this.fontSize = WidgetRegisterGrid.REFERENCE_FONT_SIZE;
    this.registerModelIdMap = new Map();
    this.onSelectedPathChanged = () => {
      var _a, _b;
      this.selectedRegisterPath = (_b = (_a = this.grid) === null || _a === void 0 ? void 0 : _a.selectedPath) !== null && _b !== void 0 ? _b : "";
      const segments = this.selectedRegisterPath.split("/");
      if (segments.length > 1) {
        this.selectedRegister = segments[1];
      } //else {
      //   this.selectedRegister = "";
      // }
    };
    this.onRegisterValueChangedListener = () => {
      var _a;
      (_a = this.grid) === null || _a === void 0 ? void 0 : _a.refresh();
    };
    this.onRegisterSymbolsChangedListener = () => {
      var _a, _b;
      (_a = this.dataProvider) === null || _a === void 0 ? void 0 : _a.removeChangeListeners(this.onRegisterValueChangedListener);
      this.dataProvider = new RegisterGridDataProvider(this.registerModel, this.virtual);
      (_b = this.grid) === null || _b === void 0 ? void 0 : _b.setDataProvider(this.dataProvider, this.getExpandToLevel(), this.registerModel ? this.registerModelId : undefined);
      this.dataProvider.addChangeListeners(this.onRegisterValueChangedListener);
    };
    this.dataProvider = undefined;
    this.selectedRegisterPath = "";
    this.registerModelId = undefined;
    this.addressBits = undefined;
    this.dataBits = undefined;
    this.virtual = false;
    this.selectedField = undefined;
    this.selectedRegister = "";
    this.filterText = undefined;
    this.sortBy = undefined;
    this.sortDescending = false;
    this.hideAddressColumn = false;
    this.hideBitsColumn = false;
    this._refresh = 0;
    this.hidden = false;
  }
  onVirtualChanged() {
    this.onRegisterSymbolsChangedListener();
  }
  getExpandToLevel() {
    if (this.registerModel) {
      if (this.registerModelIdMap.has(this.registerModelId)) {
        return 1;
      }
      this.registerModelIdMap.set(this.registerModelId, true);
    }
    return 2;
  }
  async addRegisterDefinitionsListener() {
    if (this.registerModelId) {
      this.registerModel = bindingRegistry.getModel(this.registerModelId);
      if (!this.registerModel) {
        const modelId = this.registerModelId;
        await codecRegistry.whenConfigurationReady(this.registerModelId);
        if (this.registerModelId !== modelId) {
          return;
        }
        this.registerModel = bindingRegistry.getModel(this.registerModelId);
      }
    }
    this.onRegisterSymbolsChangedListener();
    if (this.registerModel) {
      this.registerModel
        .getBinding("$registerDefinitions")
        .addEventListener(valueChangedEventType, this.onRegisterSymbolsChangedListener);
    }
  }
  removeRegisterDefinitionsListener() {
    if (this.registerModel) {
      this.registerModel
        .getBinding("$registerDefinitions")
        .removeEventListener(valueChangedEventType, this.onRegisterSymbolsChangedListener);
    }
    this.registerModel = undefined;
  }
  get addressPrecision() {
    var _a;
    return this.addressBits
      ? Math.ceil(this.addressBits / 4)
      : ((_a = this.dataProvider) === null || _a === void 0 ? void 0 : _a.addressSize) || 2;
  }
  get displayBits() {
    var _a;
    return this.dataBits || ((_a = this.dataProvider) === null || _a === void 0 ? void 0 : _a.dataBits) || 8;
  }
  get dataPrecision() {
    return Math.ceil(this.displayBits / 4);
  }
  componentWillRender() {
    const fontSize = Number.parseFloat(getComputedStyle(this.el).getPropertyValue("font-size"));
    if (!isNaN(fontSize)) {
      this.fontSize = fontSize;
    }
  }
  render() {
    var _a;
    const valueWidth = Math.ceil(((Math.max(this.dataPrecision, 8) + 2) * 10 * this.fontSize) /
      WidgetRegisterGrid.REFERENCE_FONT_SIZE);
    let selectedBitsMask = 0;
    if (this.selectedField && this.selectedRegister) {
      const fieldInfo = (_a = this.registerModel) === null || _a === void 0 ? void 0 : _a.getRegisterInfo(`${this.selectedRegister}.${this.selectedField}`);
      if (fieldInfo) {
        selectedBitsMask = GcUtils.bitField.getMask(fieldInfo.start, fieldInfo.stop);
      }
    }
    return (h("gc-widget-grid", { id: "grid", ref: (el) => (this.grid = el), selectedPath: this.selectedRegisterPath, "onSelected-path-changed": this.onSelectedPathChanged, filterText: this.filterText, sortByColumn: this.sortBy, sortDescending: this.sortDescending, virtual: this.virtual }, this.hideAddressColumn ? null : (h("gc-widget-grid-data-column", { key: "address", heading: "Address", name: "address", readonly: true, format: "hex", precision: this.addressPrecision, hideMinimizeAction: true })), h("gc-widget-grid-tree-column", { heading: "Register Name", hideMinimizeAction: true, showInfoIcon: true }, h("slot", null)), h("gc-widget-grid-data-column", { key: "value", heading: "Value", name: "value", format: "hex", precision: this.dataPrecision, style: { "--gc-column-width": `${valueWidth}px` }, hideMinimizeAction: true }), this.hideBitsColumn ? null : (h("gc-widget-register-bits-column", { id: "bitsHeader", heading: "Bits", name: "value", dataBits: this.displayBits, selectedBitsMask: selectedBitsMask, hideMinimizeAction: true }))));
  }
  onRegisterModelIdChanged() {
    this.removeRegisterDefinitionsListener();
    this.addRegisterDefinitionsListener();
  }
  onSelectedRegisterChanged() {
    var _a, _b;
    // push selected register change to the grid.  If the change came from the grid originally, this should have no effect.
    if (this.selectedRegister) {
      (_a = this.dataProvider) === null || _a === void 0 ? void 0 : _a.lookupRegisterByName(this.selectedRegister);
      // this.selectedRegisterPath = registerData?.path ?? "";
    } // else if (this.selectedRegisterPath.split("/").length > 1) {
    //   // We don't want to remove a selected group, so we need to make sure the path includes a register name to be cleared.
    //   this.selectedRegisterPath = "";
    // }
    this.selectedRegisterChanged.emit({
      value: (_b = this.dataProvider) === null || _b === void 0 ? void 0 : _b.lookupRegisterByName(this.selectedRegister).binding.name,
    });
  }
  /**
   * Helper method to find a the corresponding bit field, of the selected register, from a single bit of the register.
   *
   * @param bit zero based index of the bit to find the corresponding bit field for.
   */
  async lookupFieldByBit(bit) {
    var _a, _b;
    const symbolInfo = (_a = this.registerModel) === null || _a === void 0 ? void 0 : _a.getRegisterInfo(this.selectedRegister);
    const fieldInfo = (_b = symbolInfo === null || symbolInfo === void 0 ? void 0 : symbolInfo.fields) === null || _b === void 0 ? void 0 : _b.find(field => field.start >= bit && field.stop <= bit);
    return (fieldInfo === null || fieldInfo === void 0 ? void 0 : fieldInfo.id) || (fieldInfo === null || fieldInfo === void 0 ? void 0 : fieldInfo.name);
  }
  /**
   * Method to set or change the filter function for filtering registers in the grid.  If no callback is
   * provided, the default filtering will be performed on the register name.
   *
   * @param filterGridByRowCallback callback method that returns true if the row should remain after filtering.
   */
  async setFilterMethod(filterGridByRowCallback) {
    var _a;
    (_a = this.grid) === null || _a === void 0 ? void 0 : _a.setFilterMethod(filterGridByRowCallback);
  }
  /**
   * Method to redraw the underlying grid.  Calling refresh() may not cause the data in the grid to
   * redraw, so this method can be used to force a redrawing of the grid data.
   */
  async redrawGrid() {
    var _a;
    (_a = this.grid) === null || _a === void 0 ? void 0 : _a.refresh();
  }
  /**
   * Method to expand all register groups to a specified level.
   *
   * @param level depth or level expanded register groups.
   */
  expandToLevel(level) {
    if (!this.grid) {
      throw new Error("Register grid not rendered yet.  You cannot expand levels until the grid has rendered at least once.");
    }
    return this.grid.expandToLevel(level);
  }
  connectedCallback() {
    this.addRegisterDefinitionsListener();
  }
  disconnectedCallBack() {
    this.removeRegisterDefinitionsListener();
  }
  componentDidLoad() {
    var _a;
    if (this.dataProvider) {
      // if we already have our first dataProvider, then we missed setting it on the grid, since the grid is only available
      // after the first render, which is now.
      (_a = this.grid) === null || _a === void 0 ? void 0 : _a.setDataProvider(this.dataProvider, 2, this.registerModel ? this.registerModelId : undefined);
    }
  }
  /**
   * Sets the CSS property.
   *
   * @param {string} name the element style name
   * @param {string} value the new CSS property to be set
   */
  async setCSSProperty(name, value) {
    value = value.trim();
    if ((await this.getCSSProperty(name)) !== value) {
      this.el.style.setProperty(name, value);
      this.cssPropertyChanged.emit({ name: name, value: value });
    }
  }
  /**
   * Returns the value of a CSS property.
   *
   * @param {string} name the element style property
   * @returns {string} the value of the property
   */
  async getCSSProperty(name) {
    return getComputedStyle(this.el).getPropertyValue(name).trim();
  }
  /**
   * Refresh the widget.
   */
  async refresh() {
    this._refresh++;
  }
  get el() { return getElement(this); }
  static get watchers() { return {
    "virtual": ["onVirtualChanged"],
    "registerModelId": ["onRegisterModelIdChanged"],
    "selectedRegister": ["onSelectedRegisterChanged"]
  }; }
};
WidgetRegisterGrid.REFERENCE_FONT_SIZE = 14;
WidgetRegisterGrid.style = gcCustomRegisterGridCss;

export { WidgetRegisterBitfieldViewer as gc_custom_register_bitfield_viewer, WidgetRegisterGrid as gc_custom_register_grid };

//# sourceMappingURL=p-ad8e52ae.entry.js.map